import _classPrivateFieldLooseBase from "@babel/runtime/helpers/esm/classPrivateFieldLooseBase";
import _classPrivateFieldLooseKey from "@babel/runtime/helpers/esm/classPrivateFieldLooseKey";
// Copyright 2021 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
import { mnemonicToMiniSecret } from '@polkadot/util-crypto';
import { File } from '@skyekiwi/file';
import { fromBase64, hexToU8a, isValidHex, isValidSubstrateAddress, stringToU8a, toBase64, u8aToHex, u8aToString } from '@skyekiwi/util';

var _file = /*#__PURE__*/_classPrivateFieldLooseKey("file");

var _sealer = /*#__PURE__*/_classPrivateFieldLooseKey("sealer");

var _contract = /*#__PURE__*/_classPrivateFieldLooseKey("contract");

var _seed = /*#__PURE__*/_classPrivateFieldLooseKey("seed");

export class SContractReader {
  constructor(file, sealer) {
    Object.defineProperty(this, _file, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _sealer, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _contract, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _seed, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldLooseBase(this, _file)[_file] = file;
    _classPrivateFieldLooseBase(this, _sealer)[_sealer] = sealer;
  }

  async init() {
    try {
      const fullContract = await _classPrivateFieldLooseBase(this, _file)[_file].readAll();
      _classPrivateFieldLooseBase(this, _contract)[_contract] = this.decodeContract(fromBase64(fullContract));
    } catch (err) {
      throw new Error('initialization error, file might not be found s-contract/SContractReader/init');
    }
  }

  unlockSealer(seed) {
    const key = mnemonicToMiniSecret(seed);

    _classPrivateFieldLooseBase(this, _sealer)[_sealer].unlock(key);
  }

  encodeContract(seed, contract) {
    const authString = contract.auth.map(SContractSerde.encodeAuth).join('|');
    const result = `${contract.contractId}*${seed}*${authString}*${contract.highLocalCallIndex}*${contract.state}*${contract.wasmPath}`;
    return result;
  }

  decodeContract(contractRaw) {
    const contract = contractRaw.split('*');
    const contractId = contract[0];
    const seed = contract[1];
    const auths = contract[2].split('|').map(SContractSerde.decodeAuth);
    const highLocalCallIndex = contract[3];
    const state = contract[4];
    const wasmPath = contract[5];
    this.unlockSealer(seed);
    _classPrivateFieldLooseBase(this, _seed)[_seed] = seed;
    return {
      auth: auths,
      contractId: contractId,
      highLocalCallIndex: highLocalCallIndex,
      state: state,
      wasmPath: wasmPath
    };
  }

  writeState(state) {
    _classPrivateFieldLooseBase(this, _contract)[_contract].state = state;
  }

  readState() {
    return _classPrivateFieldLooseBase(this, _contract)[_contract].state;
  }

  writeAuth(auth) {
    _classPrivateFieldLooseBase(this, _contract)[_contract].auth.push(auth);
  }

  revokeAuth(auth) {
    _classPrivateFieldLooseBase(this, _contract)[_contract].auth.forEach((v, i) => {
      if (v === auth) {
        _classPrivateFieldLooseBase(this, _contract)[_contract].auth.splice(i, 1);
      }
    });
  }

  getContractId() {
    return _classPrivateFieldLooseBase(this, _contract)[_contract].contractId;
  }

  async writeToFile(path) {
    await File.writeFile(stringToU8a(toBase64(this.encodeContract(_classPrivateFieldLooseBase(this, _seed)[_seed], _classPrivateFieldLooseBase(this, _contract)[_contract]))), path, 'w');
  }

  forceInject(seed, contract) {
    _classPrivateFieldLooseBase(this, _contract)[_contract] = contract;
    _classPrivateFieldLooseBase(this, _seed)[_seed] = seed;

    _classPrivateFieldLooseBase(this, _sealer)[_sealer].unlock(mnemonicToMiniSecret(seed));
  }

  getHighLocalCallIndex() {
    return _classPrivateFieldLooseBase(this, _contract)[_contract].highLocalCallIndex;
  }

  readContract() {
    console.log(_classPrivateFieldLooseBase(this, _contract)[_contract]);
    return _classPrivateFieldLooseBase(this, _contract)[_contract];
  }

}
export class SContractSerde {
  static decodeCall(call, sealer) {
    const _call = call.split('?');

    let callContent = '';
    const callIndex = _call[0];
    let encrypted = false;

    if (isValidHex(_call[1])) {
      encrypted = true;
      callContent = u8aToString(sealer.decrypt(hexToU8a(call)));
    } else {
      callContent = _call[1];
    }

    const oneCall = callContent.split('?');
    return {
      callIndex: callIndex,
      contractId: oneCall[0],
      encrypted: encrypted,
      methodName: fromBase64(oneCall[1]),
      origin: fromBase64(oneCall[2]),
      parameters: fromBase64(oneCall[3])
    };
  }

  static encodeCall(call, sealer) {
    if (call.methodName.length < 0 || call.methodName.length > 32) {
      throw new Error('methodName must be between 0 - 32 bytes - calls/encodeCall');
    }

    if (call.parameters.length < 0 || call.parameters.length > 128) {
      throw new Error('parameters must be between 0 - 128 bytes - calls/encodeCall');
    }

    if (!isValidSubstrateAddress(call.origin)) {
      throw new Error('origin must be a valid Substrate address - calls/encodeCall');
    }

    let callString = `${call.contractId}?${toBase64(call.methodName)}?${toBase64(call.origin)}?${toBase64(call.parameters)}`;

    if (call.encrypted) {
      callString = u8aToHex(sealer.encrypt(stringToU8a(callString), sealer.getAuthorKey()));
    }

    return call.callIndex + '?' + callString;
  }

  static encodeAuth(auth) {
    return toBase64(`${auth.storageKey}?${auth.authOrigin}`);
  }

  static decodeAuth(authStringRaw) {
    const authString = fromBase64(authStringRaw);
    const a = authString.split('?');
    const storageKey = a[0];
    const auth = a[1];
    if (storageKey === undefined) return null;
    return {
      authOrigin: auth,
      storageKey: storageKey
    };
  }

}